private import  mango.io.Stdout;
private import  mango.io.File;
private import  mango.io.FilePath;
private import  mango.io.FileProxy;

private import  mango.utils.Text;

private import  example.Processor;
private import  example.BlockParser;

/*******************************************************************************
        
        text replacements

*******************************************************************************/

SimplePatch[] Compatibility = 
[
{"public static double PI = Math.PI;", "public static const double PI = Math.PI;"},
{"static double toRadians = PI / 180;", "static const double toRadians = PI / 180;"},
];

SimplePatch[] ImageData = 
[
{"new ImageLoader().load(stream);", "(new ImageLoader()).load(stream);"},
{"new ImageLoader().load(filename);", "(new ImageLoader()).load(filename);"},
{"(flipX) ?", "flipX ?"},
{"(flipY) ?", "flipY ?"},
{"ANY_TO_EIGHT = new byte[9][];", "ANY_TO_EIGHT = new byte[][9];"},
{"byte [] masks = { cast(byte)0xFC, cast(byte)0xF3, cast(byte)0xCF, cast(byte)0x3F };", "byte [] masks = ConstructArray!(byte)(cast(byte)0xFC, cast(byte)0xF3, cast(byte)0xCF, cast(byte)0x3F);"},
{"final int", "const int"},
{"paletteData = new PaletteData(0x0000ff00, 0x00ff0000, 0xff000000);", "paletteData = new PaletteData(0x0000ff00, 0x00ff0000, cast(int) 0xff000000);"},
];

SimplePatch[] Image = 
[
{"int hdcMask = OS.CreateCompatibleDC(hDC);", "int hdcMask = OS.CreateCompatibleDC(hDC); int hOldSrc, hOldBw, hOldBmp, hb, oldBrush, bm; RECT rect;"},
{"blueMask = 0xFF000000;", "blueMask = cast(int) 0xFF000000;"},
{"\tbm = new BITMAP()", "\tBITMAP bm = new BITMAP()"},
{"palette = new PaletteData(0xFF00, 0xFF0000, 0xFF000000);", "palette = new PaletteData(0xFF00, 0xFF0000, cast(int) 0xFF000000);"},
];

SimplePatch[] Device = 
[
{"public static bool DEBUG;", "public static const bool DEBUG;"},
{"scripts = new int [piNumScripts [0]];", "scripts = new int [(piNumScripts [0])];"},
];

SimplePatch[] OS = 
[
{"static final int SYS_COLOR_INDEX_FLAG = OS.IsWinCE ? 0x40000000 : 0x0;", "version (WinCE) static const int SYS_COLOR_INDEX_FLAG = 0x40000000; else static const int SYS_COLOR_INDEX_FLAG = 0;"},
//{"public static final int NOTIFYICONDATA_V2_SIZE = IsUnicode ? NOTIFYICONDATAW_V2_SIZE : NOTIFYICONDATAA_V2_SIZE;", "// public static final int NOTIFYICONDATA_V2_SIZE = IsUnicode ? NOTIFYICONDATAW_V2_SIZE : NOTIFYICONDATAA_V2_SIZE;"},
{"OSVERSIONINFO.sizeof = info.dwOSVersionInfoSize;", "//OSVERSIONINFO._sizeof = info.dwOSVersionInfoSize;"},
{"public static final native int AbortDoc (int hdc);", "extern (Windows) {\npublic static int AbortDoc (int hdc);"},
{"public static final native int wcslen (int string);", "public static int wcslen (int string);\n}"},
{"IsSP()", "_IsSP()"},
{"IsPPC()", "_IsPPC()"},
];

SimplePatch[] FileFormat = 
[
{"catch (Exception e) ", "catch (Throwable e) "},
];

SimplePatch LEDataOutputStream = {"read(buf)", "super.read(buf);"};

SimplePatch[] OSVERSIONINFO = 
[
{"int sizeof = OS.IsUnicode ? 276 : 148;", "public static int sizeof; static this(){sizeof = OS.IsUnicode ? 276 : 148;}"},
{"public static", ""},
];

/+
public abstract class FileFormat {
	static String FORMAT_PACKAGE; //$NON-NLS-1$
	static String FORMAT_SUFFIX; //$NON-NLS-1$
	static String[] FORMATS = {String.create("WinBMP"), String.create("WinBMP"), String.create("GIF"), String.create("WinICO"), String.create("JPEG"), String.create("PNG"), String.create("TIFF")}; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$//$NON-NLS-5$ //$NON-NLS-6$//$NON-NLS-7$
 +/

char[] x = 
"	}
	static final byte[] ONE_TO_ONE_MAPPING = ANY_TO_EIGHT[8];

	/**
	 * Scaled 8x8 Bayer dither matrix.
	 */
	static final int[][] DITHER_MATRIX = {
		{ 0xfc0000, 0x7c0000, 0xdc0000, 0x5c0000, 0xf40000, 0x740000, 0xd40000, 0x540000 },
		{ 0x3c0000, 0xbc0000, 0x1c0000, 0x9c0000, 0x340000, 0xb40000, 0x140000, 0x940000 },
		{ 0xcc0000, 0x4c0000, 0xec0000, 0x6c0000, 0xc40000, 0x440000, 0xe40000, 0x640000 },
		{ 0x0c0000, 0x8c0000, 0x2c0000, 0xac0000, 0x040000, 0x840000, 0x240000, 0xa40000 },
		{ 0xf00000, 0x700000, 0xd00000, 0x500000, 0xf80000, 0x780000, 0xd80000, 0x580000 },
		{ 0x300000, 0xb00000, 0x100000, 0x900000, 0x380000, 0xb80000, 0x180000, 0x980000 },
		{ 0xc00000, 0x400000, 0xe00000, 0x600000, 0xc80000, 0x480000, 0xe80000, 0x680000 },
		{ 0x000000, 0x800000, 0x200000, 0xa00000, 0x080000, 0x880000, 0x280000, 0xa80000 }
	};";

char[] y = 
"	ONE_TO_ONE_MAPPING = ANY_TO_EIGHT[8];
	}
        static final byte[] ONE_TO_ONE_MAPPING;
	/**
	 * Scaled 8x8 Bayer dither matrix.
	 */
	static final int[][] DITHER_MATRIX = [
		[ 0xfc0000, 0x7c0000, 0xdc0000, 0x5c0000, 0xf40000, 0x740000, 0xd40000, 0x540000 ],
		[ 0x3c0000, 0xbc0000, 0x1c0000, 0x9c0000, 0x340000, 0xb40000, 0x140000, 0x940000 ],
		[ 0xcc0000, 0x4c0000, 0xec0000, 0x6c0000, 0xc40000, 0x440000, 0xe40000, 0x640000 ],
		[ 0x0c0000, 0x8c0000, 0x2c0000, 0xac0000, 0x040000, 0x840000, 0x240000, 0xa40000 ],
		[ 0xf00000, 0x700000, 0xd00000, 0x500000, 0xf80000, 0x780000, 0xd80000, 0x580000 ],
		[ 0x300000, 0xb00000, 0x100000, 0x900000, 0x380000, 0xb80000, 0x180000, 0x980000 ],
		[ 0xc00000, 0x400000, 0xe00000, 0x600000, 0xc80000, 0x480000, 0xe80000, 0x680000 ],
		[ 0x000000, 0x800000, 0x200000, 0xa00000, 0x080000, 0x880000, 0x280000, 0xa80000 ]
	];";


/*******************************************************************************
        
        Example regex: 

*******************************************************************************/

class NullAssignment : CustomPatch
{
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "custom null assignments patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"(((char|byte)\s*\[\])[^\?;]+\?) null");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add (" cast(");
                            add (g3);
                            add ("[]) null");
                            eat ();
                            }
                return combine;
        }
}

/*******************************************************************************
        
*******************************************************************************/

class OSConstants : CustomPatch
{
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "custom constants patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"(static final int [\w_]+\s*=\s*)");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add ("cast(int) ");
                            eat ();
                            }
                return combine;
        }
}


/*******************************************************************************
        
        Example mechanism to override or augment the default processing

*******************************************************************************/

class MyProcessor : Processor
{
        /***********************************************************************

        ***********************************************************************/

        this (FilePath path)
        {
                super (path);
        }

        /***********************************************************************

        ***********************************************************************/

        protected bool patchPath (MutableFilePath dst)
        {
                int i = Text.indexOf (dst.getPath, "eclipse");
                if (i > 0)
                   {
                   dst.setRoot (null).setPath(dst.getPath()[i+8..length]);
                   return true;
                   }
                return false;
        }

        /***********************************************************************

        ***********************************************************************/

        protected bool directory (FilePath path)
        {
                MutableFilePath dst = new MutableFilePath (path);
                if (patchPath (dst))
                   {
                   FileProxy fp = new FileProxy (dst);
                   if (! fp.isExisting)
                         fp.createDirectory();
                   }
                return true;
        }

        /***********************************************************************

                Convert input-file path to an output file

        ***********************************************************************/

        protected void nameOutputFile (MutableFilePath dst)
        {
                super.nameOutputFile (dst);
                patchPath (dst);
        }

        /***********************************************************************
        
                Perform general text replacements: all code, all files

        ***********************************************************************/

        protected void general (Parser.Node node)
        {
                super.general (node);
        }
}

/*******************************************************************************

        Create a custom converter, and register custom handlers

*******************************************************************************/

void main (char[][] args)
{       
        if (args.length == 2)
           {
           // instantiate the converter
           MyProcessor processor = new MyProcessor (new FilePath (FilePath.normalize(args[1])));

           // custom conversions 
//           processor.replace ("swt/SWT.java", new Constants);

           processor.replace ("swt/internal/win32/OS.java", new NullAssignment);
           processor.replace ("swt/internal/win32/OS.java", new OSConstants);
           processor.replace ("swt/internal/win32/OS.java", OS);

           processor.replace ("swt/internal/Compatibility.java", Compatibility);

           processor.replace ("swt/graphics/Device.java", Device);

           processor.replace ("swt/graphics/ImageData.java", ImageData);

           processor.replace ("swt/graphics/internal/image/LEDataOutputStream.java", LEDataOutputStream);

           processor.replace ("swt/internal/image/FileFormat.java", FileFormat);

           processor.replace ("swt/internal/image/LEDataOutputStream.java", LEDataOutputStream);

           processor.replace ("swt/internal/win32/OSVERSIONINFO.java", OSVERSIONINFO);

           // This file will be replaced completely; don't include in imports
           processor.ignore ("swt/internal/Callback.java");

           // start conversion
           processor.start();
           }
        else
           printf ("usage: swt directory\n");
}



