private import  mango.io.Stdout;
private import  mango.io.File;
private import  mango.io.FilePath;
private import  mango.io.FileProxy;

private import  mango.utils.Text;

private import  example.Processor;
private import  example.BlockParser;

/*******************************************************************************
        
        text replacements

*******************************************************************************/

Processor.SimplePatch[] Compatibility = 
[
{"public static double PI = Math.PI;", "public static const double PI = Math.PI;"},
{"static double toRadians = PI / 180;", "static const double toRadians = PI / 180;"},
];

/*******************************************************************************
        
        Example regex: 

*******************************************************************************/

class Constants : CustomPatch
{
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "constant types patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this (char[] type)
        {
                super (r"(((final|static)\s+)+)(" ~ type ~ r"\s+[\w_]+\s*=)");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add ("const ");
                            add (g4);
                            eat ();
                            }
                return combine;
        }
}


/*******************************************************************************
        
        Example mechanism to override or augment the default processing

*******************************************************************************/

class MyProcessor : Processor
{
        /***********************************************************************

        ***********************************************************************/

        this (FilePath path)
        {
                super (path);
        }

        /***********************************************************************

        ***********************************************************************/

        protected bool patchPath (MutableFilePath dst)
        {
                int i = Text.indexOf (dst.getPath, "eclipse");
                if (i > 0)
                   {
                   dst.setRoot (null).setPath(dst.getPath()[i+8..length]);
                   return true;
                   }
                return false;
        }

        /***********************************************************************

        ***********************************************************************/

        protected bool directory (FilePath path)
        {
                MutableFilePath dst = new MutableFilePath (path);
                if (patchPath (dst))
                   {
                   FileProxy fp = new FileProxy (dst);
                   if (! fp.isExisting)
                         fp.createDirectory();
                   }
                return true;
        }

        /***********************************************************************

                Convert input-file path to an output file

        ***********************************************************************/

        protected void nameOutputFile (MutableFilePath dst)
        {
                super.nameOutputFile (dst);
                patchPath (dst);
        }

        /***********************************************************************
        
                Perform general text replacements: all code, all files

        ***********************************************************************/

        protected void general (Parser.Node node)
        {
                super.general (node);
        }
}

/*******************************************************************************

        Create a custom converter, and register custom handlers

*******************************************************************************/

void main (char[][] args)
{       
        if (args.length == 2)
           {
           // instantiate the converter
           MyProcessor processor = new MyProcessor (new FilePath (FilePath.normalize(args[1])));

           // custom conversions 
           processor.replace ("swt/SWT.java", new Constants ("int"));
           processor.replace ("swt/internal/Compatibility.java", Compatibility);

           // start conversion
           processor.start();
           }
        else
           printf ("usage: swt directory\n");
}



