/*******************************************************************************

        @file File.d
        
        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version; March 2005      

        @author         Kris


*******************************************************************************/

module example.File;

public  import  mango.io.FileStyle;

private import  mango.io.FileConduit;

/*******************************************************************************

*******************************************************************************/

class File
{
        private Buffer          buffer;
        private FileConduit     conduit;

        /***********************************************************************

        ***********************************************************************/

        this (char[] name, FileStyle style = FileStyle.ReadWriteExisting)
        {
                this (new FilePath(name), style);
        }

        /***********************************************************************

        ***********************************************************************/

        this (FilePath path, FileStyle style = FileStyle.ReadWriteExisting)
        {
                // setup to read file
                conduit = new FileConduit (path, style);  
                uint length = conduit.length;
                buffer = new Buffer (length);
                buffer.setConduit (conduit);

                // read the entire file into memory; don't eat the content yet
                buffer.get (length, false);
        }

        /***********************************************************************

        ***********************************************************************/

        ~this ()
        {
                close ();
        }

        /***********************************************************************

        ***********************************************************************/

        void close ()
        {
                conduit.close ();
        }

        /***********************************************************************

        ***********************************************************************/

        void[] get ()
        {
                // return whatever is currently in the buffer
                return buffer.toString ();
        }

        /***********************************************************************

        ***********************************************************************/

        FileConduit getConduit ()
        {
                return conduit;
        }

        /***********************************************************************

        ***********************************************************************/

        IBuffer getBuffer ()
        {
                return buffer;
        }

        /***********************************************************************

        ***********************************************************************/

        void set (void[] content)
        {
                // set file content equal to provided content
                conduit.seek (0);
                buffer.setValidContent (content);
                buffer.flush ();

                // set file size
                conduit.seek (content.length);
                conduit.truncate ();               

                // allow get() to still see this content
                buffer.setValidContent (content);
        }
}