module example.Server;

/*******************************************************************************

*******************************************************************************/

alias void delegate (Page page) PageHandler;
alias void delegate (Server server) RequestHandler;
alias void delegate (List list, IWriter writer) ListHandler;
alias void delegate (Element element, IWriter writer) ElementHandler;


/*******************************************************************************

*******************************************************************************/

interface IWriter
{
        alias put opCall;

        IWriter put (char[] x);
}


/*******************************************************************************

*******************************************************************************/

interface IReader
{
        alias get opCall;

        IReader get (char[] x);
}


/*******************************************************************************

*******************************************************************************/

class Navigator 
{
        private Navigator parent;

        /***********************************************************************

        ***********************************************************************/

        this (Navigator parent)
        {
                this.parent = parent;        
        }

        /***********************************************************************

        ***********************************************************************/

        abstract void begin (Server server);

        /***********************************************************************

        ***********************************************************************/

        abstract bool isStateful ();

        /***********************************************************************

                Create a fresh instance of this object

        ***********************************************************************/

        abstract Navigator create ();
}


/*******************************************************************************

*******************************************************************************/

class Stateful : Navigator
{
        /***********************************************************************

        ***********************************************************************/

        this (Navigator parent)
        {
                super (parent);
        }

        /***********************************************************************

                Save the current state

        ***********************************************************************/

        abstract void read (IReader read);

        /***********************************************************************

                Restore the current state

        ***********************************************************************/

        abstract void write (IWriter write);

        /***********************************************************************

        ***********************************************************************/

        bool isStateful ()
        {
                return true;
        }
}


/*******************************************************************************

*******************************************************************************/

class Stateless : Navigator
{
        /***********************************************************************

        ***********************************************************************/

        this (Navigator parent)
        {
                super (parent);
        }

        /***********************************************************************

        ***********************************************************************/

        bool isStateful ()
        {
                return false;
        }
}


/*******************************************************************************

*******************************************************************************/

class NamedContent 
{
        int getID();
        char[] getName();
}


/*******************************************************************************

*******************************************************************************/

class Element : NamedContent
{
}


/*******************************************************************************

*******************************************************************************/

class List : NamedContent
{
        void populate (IWriter writer, ElementHandler elements)
        {
        }
}


/*******************************************************************************

*******************************************************************************/

class Page : NamedContent
{
}


/*******************************************************************************

*******************************************************************************/

class View
{
        /***********************************************************************

        ***********************************************************************/

        abstract char[] getResource ();

        /***********************************************************************

        ***********************************************************************/

        char[] getCacheKey ()
        {
                return null;
        }

        /***********************************************************************

        ***********************************************************************/

        void populate (IWriter w, Server s)
        {
        }

        /***********************************************************************

        ***********************************************************************/

        bool populate (IWriter w, Element e)
        {
                return false;
        }

        /***********************************************************************

        ***********************************************************************/

        bool populate (IWriter w, List l)
        {
                return false;
        }

        /***********************************************************************

        ***********************************************************************/

        bool populate (IWriter w, Page p)
        {
                return false;
        }
}


/*******************************************************************************

*******************************************************************************/

class ClientArgs 
{
}


/*******************************************************************************

*******************************************************************************/

class Server 
{
        bool reply (View view, RequestHandler rh = null);
        bool reply (char[] page, RequestHandler rh = null);

        Page getPage (char[] name);

        void getArgument (char[] name, inout char[] dst)
        {
        }

        static void register (char[] pattern, Navigator function(Server) f)
        {
        }
}


