private import  std.ctype;

private import  mango.utils.Text;

private import  mango.io.Token,
                mango.io.Stdout,
                mango.io.Tokenizer,
                mango.io.FileSystem,
                mango.io.FileConduit;

/*******************************************************************************

        Open a file and read it line-by-line using a line tokenizer. This 
        uses CompositeToken to hide some of the details. 

*******************************************************************************/

void main (char[][] args)
{
	if (args.length == 2)
	   {
           // open a file for reading
           FileConduit fc = new FileConduit (args[1]);

           // create a Token and bind it to both the file and a line-tokenizer
           CompositeToken tok = new CompositeToken (Tokenizers.line, fc);
           Format format = new Format (tok.getBuffer, Stdout.getBuffer, tok);
           while (format.next (0)) {};
           Stdout ();
           }
        else
           Stdout ("usage: format filename") (CR);
}


/*******************************************************************************

*******************************************************************************/

class Format 
{
        private IBuffer         src,
                                dst;
        private CompositeToken  line;

        private static bool[char[]] KeyWords;
        private static char[256] Indentation;

        private enum State {Normal, SQuote, DQuote, Word, Paren, Block, Comment, Comment1, Comment2};
        private State state = State.Normal;

        /***********************************************************************

        ***********************************************************************/

        static this()
        {
                Indentation[] = ' ';
                KeyWords["if"] = true;
                KeyWords["while"] = true;
                KeyWords["for"] = true;
                KeyWords["else"] = true;
                KeyWords["do"] = true;
        }

        /***********************************************************************

        ***********************************************************************/

        this (IBuffer src, IBuffer dst, CompositeToken line) 
        {
                this.src = src;
                this.dst = dst;
                this.line = line;
        }

        /***********************************************************************

        ***********************************************************************/

        void emit (char[] content, int level) 
        {
                // emit the content plus a CR
                dst.append (Indentation [0 .. level]).append(content).append(FileSystem.NewlineString);
        }

        /***********************************************************************

        ***********************************************************************/

        bool next (int level, char terminator = '}')
        {
                while (line.get)
                      {
                      int       mark, 
                                indent;
                      char      lastChar;
                      
                      char[] content = (state >= State.Comment) ? line.toString : Text.trim (line.toString);
                      emit (content, level);

                      for (int i; i < content.length; ++i)
                          {
                          char c = content.ptr[i];
                          switch (state)
                                 {
                                 case State.SQuote:
                                      if (c == '\'' && lastChar != '\\')
                                          state = State.Normal;
                                      break;

                                 case State.DQuote:
                                      if (c == '"' && lastChar != '\\')
                                          state = State.Normal;
                                      break;

                                 case State.Word:
                                      if (! (isalnum(c) || c == '.' || c == '_'))
                                         {
                                         // printf ("%.*s\n", content[mark..i]);
                                         if (content[mark..i] in KeyWords)
                                            {
                                            state = State.Normal;
//                                            state = State.Paren;
                                            }
                                         else
                                            state = State.Normal;

                                         // reparse last char
                                         --i;
                                         }
                                      break;

                                 case State.Paren:
                                      if (c == '(')
                                         {
                                         indent = i;
                                         state = State.Block;
                                         }
                                      break;

                                 case State.Block:
                                      if (c == '\n')
                                         {
                                         state = State.Normal;
                                         next (level + indent, '}');
                                         }
                                      break;

                                 case State.Comment:
                                      state = (c == '*') ? State.Comment1 : State.Normal;
                                      break;

                                 case State.Comment1:
                                      if (c == '*') 
                                          state = State.Comment2;
                                      break;

                                 case State.Comment2:
                                      state = (c == '/') ? State.Normal : State.Comment1;
                                      break;

                                 case State.Normal:
                                      if (c == '"')
                                          state = State.DQuote;
                                      else
                                      if (c == '\'')
                                          state = State.SQuote;
                                      else
                                      if (c == '/')
                                          state = State.Comment;
                                      else
                                      if (isalpha(c))
                                         {
                                         mark = i;
                                         state = State.Word;
                                         }
                                      else
                                      if (c == '{')
                                         {
                                         next (level + 8, '}');
                                         }
                                      else
                                      if (c == terminator)
                                         {
                                         return true;
                                         }
                                      else
                                      if (c == ',')
                                         {
                                         // mark indent for trailing commas
                                         if (indent == 0 && (content.length-i) <= 1)
                                            {
                                            next (level + mark, ';');
                                            }
                                         }
                                      break;
                                 }
                            lastChar = c;
                            }
                      }
                return false;
        }
}
