private import std.thread;

private import std.c.windows.windows;  

private import mango.sys.System;

private import mango.io.FileProxy,
               mango.io.FileStyle,
               mango.io.FileConduit;


/*******************************************************************************

        Bring in native Windows functions

*******************************************************************************/

extern (Windows) 
{
        struct STARTUPINFO
        {
                DWORD cb;
                LPTSTR lpReserved;
                LPTSTR lpDesktop;
                LPTSTR lpTitle;
                DWORD dwX;
                DWORD dwY;
                DWORD dwXSize;
                DWORD dwYSize;
                DWORD dwXCountChars;
                DWORD dwYCountChars;
                DWORD dwFillAttribute;
                DWORD dwFlags;
                WORD wShowWindow;
                WORD cbReserved2;
                LPBYTE lpReserved2;
                HANDLE hStdInput;
                HANDLE hStdOutput;
                HANDLE hStdError;
        }

        struct PROCESS_INFORMATION
        {
                HANDLE hProcess;
                HANDLE hThread;
                DWORD dwProcessId;
                DWORD dwThreadId;
        }

        BOOL CreateProcessA (
                        LPCTSTR lpApplicationName,                  // name of executable module
                        LPTSTR lpCommandLine,                      // command line string
                        LPSECURITY_ATTRIBUTES lpProcessAttributes, // SD
                        LPSECURITY_ATTRIBUTES lpThreadAttributes,  // SD
                        BOOL bInheritHandles,                      // handle inheritance option
                        DWORD dwCreationFlags,                     // creation flags
                        LPVOID lpEnvironment,                      // new environment block
                        LPCTSTR lpCurrentDirectory,                // current directory name
                        STARTUPINFO* lpStartupInfo,               // startup information
                        PROCESS_INFORMATION* lpProcessInformation // process information
                        );

        HANDLE CreateEventA (
                        LPSECURITY_ATTRIBUTES lpEventAttributes,
                        BOOL bManualReset,
                        BOOL bInitialState,
                        LPCTSTR lpName
                        );

        DWORD WaitForSingleObject (HANDLE hHandle, DWORD dwMilliseconds);

        BOOL SetEvent (HANDLE hEvent);

        BOOL ResetEvent (HANDLE hEvent);
}


/*******************************************************************************

*******************************************************************************/

class FileMonitor : Thread
{
        PROCESS_INFORMATION     pi;
        STARTUPINFO             si;
        HANDLE                  event;
        FileProxy               proxy;
        char[1]                 prefix;

        /***********************************************************************

        ***********************************************************************/

        this (char[] name)
        {
                this (name, '0');
        }

        /***********************************************************************

        ***********************************************************************/

        private this (char[] name, char prefix)
        {
                this.prefix[0] = prefix;

                proxy = new FileProxy (name);

                if (! proxy.isExisting)
                      error ("file '"~name~"' does not exist");
        }

        /***********************************************************************

        ***********************************************************************/

        void error (char[] msg)
        {
                throw new Exception (msg);
        }

        /***********************************************************************

        ***********************************************************************/

        char[] copy ()
        {
                char[] target = prefix ~ proxy.getPath.toString;

                auto FileConduit dst = new FileConduit (target, FileStyle.ReadWriteCreate);
                dst.copy (proxy.getPath);
                return target;
        }

        /***********************************************************************

        ***********************************************************************/

        void execute ()
        {
                char[] target = copy ();

                // create a named event
                char[] eventName = target ~ "\0";
                //printf ("creating event '%.*s'\n", eventName);
                event = CreateEventA (null, true, false, eventName);
                if (event == null)
                    error ("failed to create event for " ~ eventName);

                if (! CreateProcessA (target ~ "\0", null, null, null, false, 0, null, null, &si, &pi))
                      error ("failed to start child");
        }

        /***********************************************************************

        ***********************************************************************/

        override void start ()
        {
                execute ();
                super.start();
        }

        /***********************************************************************

        ***********************************************************************/

        void terminate ()
        {
                // shutdown old version
                SetEvent (event);
                WaitForSingleObject (pi.hProcess, -1);
                printf ("child terminated\n");

                // clean up child handles
                CloseHandle (pi.hProcess);
                CloseHandle (pi.hThread);
                CloseHandle (event);
        }

        /***********************************************************************

        ***********************************************************************/

        int run ()
        {
                long timestamp = proxy.getModifiedTime();

                while (true)
                      {
                      System.sleep (System.Interval.Second * 2);

                      long currentTime = proxy.getModifiedTime();
                      if (currentTime > timestamp)
                         {
                         printf ("File is newer ...\n");
                         timestamp = currentTime;
                          
                         // start new monitor, and new executable
                         char prefix = this.prefix[0];
                        if (++prefix > '9')
                            prefix = '0';
                         (new FileMonitor (proxy.getPath.toString, prefix)).start();

                         // terminate old, and exit
                         terminate ();
                         delete proxy;
                         return 0;
                         }
                      }
        }
}


/*******************************************************************************

*******************************************************************************/

void main()
{
        FileMonitor fm = new FileMonitor ("child.exe");
        fm.start();

        System.sleep ();
}


