private import  std.ctype;

private import  mango.io.Stdout;
private import  mango.io.File;
private import  mango.io.FilePath;
private import  mango.io.FileProxy;
private import  mango.io.FileConst;

private import  mango.utils.Text;

private import  example.Processor;
private import  example.BlockParser;

/*******************************************************************************

        Create a custom converter, and register custom handlers

*******************************************************************************/

void main (char[][] args)
{       
        ClassToStruct classToStruct = new ClassToStruct;

        if (args.length == 3)
           {
           // instantiate the converter
           MyProcessor processor = new MyProcessor (new FilePath (FilePath.normalize(args[1])), new FilePath (FilePath.normalize(args[2])));

           // custom conversions 
           processor.replace ("swt/SWT.java", new Constants);
        
           // This file will be replaced completely; don't include in imports
           processor.ignore ("swt/internal/Callback.java");
           processor.replace ("swt/internal/Compatibility.java", Compatibility);

           processor.replace ("swt/graphics/GC.java", GC);
           processor.replace ("swt/graphics/Image.java", Image);
           processor.replace ("swt/graphics/Image.java", new ArrayConstructor ("rgbBwBitmapInfo", "byte"));
           processor.replace ("swt/graphics/Device.java", Device);
           processor.replace ("swt/graphics/ImageData.java", ImageData);
           processor.replace ("swt/graphics/ImageData.java", new ArrayBrackets("DITHER_MATRIX"));

           processor.replace ("swt/internal/image/FileFormat.java", FileFormat);
           processor.replace ("swt/internal/image/LEDataInputStream.java", LEDataInputStream);

           processor.replace ("swt/internal/win32/OSVERSIONINFO.java", OSVERSIONINFO);
           processor.replace ("swt/internal/win32/NOTIFYICONDATA.java", NOTIFYICONDATA);

           processor.replace ("swt/widgets/Control.java", Control);
           processor.replace ("swt/widgets/Control.java", new NameClash ("fixFocus"));
           processor.replace ("swt/widgets/Listener.java", Listener);
           processor.replace ("swt/widgets/Caret.java", new NameClash("isVisible"));
           processor.replace ("swt/widgets/Decorations.java", Decorations);
           processor.replace ("swt/widgets/Shell.java", new NameClash("hwndMDIClient"));
           processor.replace ("swt/widgets/Layout.java", Layout);
           processor.replace ("swt/widgets/Table.java", Table);
           processor.replace ("swt/widgets/Tree.java", Tree);
           processor.replace ("swt/widgets/Widget.java", Widget);
           processor.replace ("swt/widgets/Dialog.java", Dialog);
           processor.replace ("swt/widgets/Synchronizer.java", Synchronizer);
           processor.replace ("swt/widgets/Display.java", new NameClash("embeddedProc"));
           processor.replace ("swt/widgets/Display.java", new NameClash("messageProc"));
           processor.replace ("swt/widgets/Display.java", new NameClash("msgFilterProc"));
           processor.replace ("swt/widgets/Display.java", new ArrayBrackets ("KeyTable"));
           processor.replace ("swt/widgets/Display.java", Display);
           processor.replace ("swt/widgets/Menu.java", Menu);
           processor.replace ("swt/widgets/Sash.java", Sash);
           processor.replace ("swt/widgets/Text.java", Text1);
           processor.replace ("swt/widgets/Shell.java", Shell);
           processor.replace ("swt/widgets/RunnableLock.java", RunnableLock);
           processor.replace ("swt/widgets/FileDialog.java", FileDialog);
           processor.replace ("swt/widgets/MessageBox.java", MessageBox);

           processor.replace ("swt/events/TypedEvent.java", TypedEvent);

           processor.replace ("swt/graphics/Cursor.java", new ArrayBrackets ("HAND_MASK"));
           processor.replace ("swt/graphics/Cursor.java", new ArrayBrackets ("HAND_SOURCE"));

           processor.replace ("swt/ole/win32/OleAutomation.java", OleAutomation);

           foreach (char[] file; ToStruct)
                    processor.replace (file, classToStruct);


           version (OS)
                   {
                   processor.replace ("swt/internal/win32/OS.java", new NullAssignment);
                   processor.replace ("swt/internal/win32/OS.java", new Constants);
                   processor.replace ("swt/internal/win32/OS.java", new OSConstants);
                   processor.replace ("swt/internal/win32/OS.java", OS);
                   }
                else
                   // this file will be replaced completely; don't include in imports
                   processor.ignore ("swt/internal/win32/OS.java");

           // start conversion
           processor.start();
           }
        else
           printf ("usage: swt input-dir output-dir\n");
}


/*******************************************************************************
        
        Example mechanism to override or augment the default processing

*******************************************************************************/

class MyProcessor : Processor
{
        FilePath        outputPath;
        FilePath        currentFile;

        NameClash       nameClash;
        StructRef       structRef;
        SimpleLiteral   simpleLiteral;

        /***********************************************************************

        ***********************************************************************/

        this (FilePath inputPath, FilePath outputPath)
        {
                super (inputPath);
                this.outputPath = outputPath;
                FileProxy fp = new FileProxy (outputPath);
                if (! fp.isExisting)
                      throw new Exception ("output path '"~outputPath.toString~"' does not exist");

                simpleLiteral = new SimpleLiteral;
                nameClash = new NameClash ("toolTipText|layout|getMsgProc|windowProc");
                structRef = new StructRef ("FILETIME|GCP_RESULTS|GRADIENT_RECT|GUITHREADINFO|HDITEM|HELPINFO|HIGHCONTRAST|ICONINFO|INITCOMMONCONTROLSEX|INPUT|KEYBDINPUT|LOGBRUSH|LOGFONT|LOGFONTA|LOGFONTW|LOGPEN|LVCOLUMN|LVHITTESTINFO|LVITEM|MARGINS|MEASUREITEMSTRUCT|MENUBARINFO|MENUINFO|MENUITEMINFO|MONITORINFO|MOUSEINPUT|MSG|NMCUSTOMDRAW|NMHDR|NMHEADER|NMLISTVIEW|NMLVCUSTOMDRAW|NMLVDISPINFO|NMLVFINDITEM|NMREBARCHEVRON|NMRGINFO|NMTOOLBAR|NMTTDISPINFO|NMTTDISPINFOA|NMTTDISPINFOW|NMTVCUSTOMDRAW|NONCLIENTMETRICS|NONCLIENTMETRICSA|NONCLIENTMETRICSW|NOTIFYICONDATA|NOTIFYICONDATAA|NOTIFYICONDATAW|OPENFILENAME|OSVERSIONINFO|OSVERSIONINFOA|OSVERSIONINFOW|PAINTSTRUCT|POINT|PRINTDLG|REBARBANDINFO|RECT|SCRIPT_ANALYSIS|SCRIPT_CONTROL|SCRIPT_FONTPROPERTIES|SCRIPT_ITEM|SCRIPT_LOGATTR|SCRIPT_PROPERTIES|SCRIPT_STATE|SCROLLINFO|SHACTIVATEINFO|SHELLEXECUTEINFO|SHMENUBARINFO|SHRGINFO|SIPINFO|SIZE|TBBUTTON|TBBUTTONINFO|TCITEM|TEXTMETRIC|TEXTMETRICA|TEXTMETRICW|TOOLINFO|TRACKMOUSEEVENT|TRIVERTEX|TVHITTESTINFO|TVINSERTSTRUCT|TVITEM|WINDOWPLACEMENT|WINDOWPOS|WNDCLASS");
        }

        /***********************************************************************

        ***********************************************************************/

        protected bool patchPath (MutableFilePath dst)
        {
                dst.setRoot (null);
                int i = Text.indexOf (dst.getPath, "eclipse");
                if (i > 0)
                   {
                   dst.setPath (outputPath.toString ~ FileConst.PathSeparatorString ~ dst.getPath[i+8..length]);
                   return true;
                   }
                else
                   dst.setPath (outputPath.toString ~ FileConst.PathSeparatorString ~ dst.getPath);
                return false;
        }

        /***********************************************************************

        ***********************************************************************/

        protected void directory (FilePath path)
        {
                MutableFilePath dst = new MutableFilePath (path);
                if (patchPath (dst))
                   {
                   FileProxy fp = new FileProxy (dst);
                   if (! fp.isExisting)
                         fp.createDirectory();
                   }
        }

        /***********************************************************************

                Convert input-file path to an output file

        ***********************************************************************/

        protected void nameOutputFile (MutableFilePath dst)
        {
                super.nameOutputFile (dst);
                patchPath (dst);
                currentFile = dst;
        }

        /***********************************************************************
        
                Perform general text replacements: all code, all files

        ***********************************************************************/

        protected void general (Parser.Node node)
        {
                super.general (node);
                node.content = structRef.convert(nameClash.convert (node.content));

                char[] p = currentFile.getPath;
        }

        /***********************************************************************

                convert this:

                new Runnable () {
                                public void run () 
                                {
                                int x = 0;
                                }
                                };      

                into this:

                new Runnable (
                             delegate void () 
                             {
                             int x = 0;
                             }
                             );

        ***********************************************************************/

        protected void processLiteral (Parser.Node node)
        {
                // printf ("literal '%.*s'\n", node.label);
                if (node.label == "Runnable" || node.label == "Listener")
                   {
                   // strip off "()"
                   node.content = node.content [0..(node.label.ptr-node.content.ptr)+node.label.length];

                   // wrap parens around whole block
                   node.bracket = "()";
                
                   // change interface method into delegate 
                   node.child.content = simpleLiteral.convert (node.child.content);
                   }
                super.processLiteral (node);
        }
}


/*******************************************************************************
        
*******************************************************************************/

class SimpleLiteral : CustomPatch
{
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "simple adapters rewritten");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"public\s+(\w+)\s+\w+(\s*\([^\)]*\))");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add ("delegate ");
                            add (g1);
                            add (g2);
                            eat ();
                            }
                return combine;
        }
}

/*******************************************************************************
        
*******************************************************************************/

class NameClash : CustomPatch
{
        private char[] nameList;

        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "name clashes patched ("~nameList~")");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this (char[] nameList)
        {
                super (r"\b("~nameList~r")\s*\W");
                this.nameList = nameList;
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                      {
                      add ();

                      char[] m = match();
                      if (m[m.length-1] != '(')
                         {
                         ++count;
                         add ("_");
                         }

                      add (m);
                      eat ();
                      }
                return combine;
        }
}


/*******************************************************************************
        
*******************************************************************************/

class StructRef : CustomPatch
{
        private char[] nameList;

        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "class references converted to structs ("~nameList~")");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this (char[] nameList)
        {
                super (r"\b("~nameList~r")\b");
                this.nameList = nameList;
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] text)
        {
                set (text);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            int i;
                            char c;
                            for (int i=end(); i < content.length; ++i)
                                 if (! isspace(c = content[i]))
                                       break;
                            
                            if (c == '{' || (
                                c == 'e' && Text.indexOf (content[i..i+6], "extends") >= 0))
                               {
                               }
                            else   
                               if (c == '[' || c == ')' || isalpha(c))
                                   add ("*");
                            eat ();
                            }
                return combine;
        }
}


/*******************************************************************************
        
*******************************************************************************/

class ArrayBrackets : CustomPatch
{
        private char[] name;

        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "array '" ~ name ~ "' patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this (char[] name)
        {
                super (r"\b" ~ name ~ r"\s*=\s*{[^;]+;");
                this.name = name;
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                      {
                      ++count;
                      char[] m = match();
                      foreach (int i, char c; m)
                               if (c == '{')
                                   m[i] = '[';
                               else
                                  if (c == '}')
                                      m[i] = ']';
                      add ();
                      add (m);
                      eat ();
                      }
                return combine;
        }
}

/*******************************************************************************
        
*******************************************************************************/

class ArrayConstructor : CustomPatch
{
        private char[] name;
        private char[] type;

        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "array constructor '" ~ name ~ "' patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this (char[] name, char[] type)
        {
                super (r"(\b" ~ name ~ r"\s*=\s*)\{([^\}]+)\}");
                this.name = name;
                this.type = type;
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add ("ConstructArray!("~type~")(");
                            add (g2);
                            add (")");
                            eat ();
                            }
                return combine;
        }
}

/*******************************************************************************
        
*******************************************************************************/

class Constants : CustomPatch
{
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "constant assignments patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"(final\s+(static\s+)*)((int|byte|char|long|bool)\s+[\w_]+\s*=)");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add ("const ");
                            add (g3);
                            eat ();
                            }
                return combine;
        }
}


/*******************************************************************************
        
*******************************************************************************/

class ClassToStruct : CustomPatch
{
        RemoveSizeof removeSizeof;

        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "classes converted to struct");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"\bclass\b");
                removeSizeof = new RemoveSizeof;
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add ("struct");
                            eat ();
                            }
                return removeSizeof (combine);
        }
}


/*******************************************************************************
        
*******************************************************************************/

class RemoveSizeof : Replace
{
        
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "sizeof members removed");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"public.* int sizeof[^;]*;");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] opCall (char[] content)
        {
                set (content);
                while (find)
                      {
                      ++count;
                      add ();
                      eat ();
                      }
                return combine;
        }
}


/*******************************************************************************
        

*******************************************************************************/

char[][] ToStruct = 
[
        "swt/internal/win32/ACCEL.java",
        "swt/internal/win32/all.java",
        "swt/internal/win32/BITMAP.java",
        "swt/internal/win32/BITMAPINFOHEADER.java",
        "swt/internal/win32/BROWSEINFO.java",
        "swt/internal/win32/CHOOSECOLOR.java",
        "swt/internal/win32/CHOOSEFONT.java",
        "swt/internal/win32/COMBOBOXINFO.java",
        "swt/internal/win32/COMPOSITIONFORM.java",
        "swt/internal/win32/CREATESTRUCT.java",
        "swt/internal/win32/DIBSECTION.java",
        "swt/internal/win32/DLLVERSIONINFO.java",
        "swt/internal/win32/DOCINFO.java",
        "swt/internal/win32/DRAWITEMSTRUCT.java",
        "swt/internal/win32/DROPFILES.java",
        "swt/internal/win32/FILETIME.java",
        "swt/internal/win32/GCP_RESULTS.java",
        "swt/internal/win32/GRADIENT_RECT.java",
        "swt/internal/win32/GUITHREADINFO.java",
        "swt/internal/win32/HDITEM.java",
        "swt/internal/win32/HELPINFO.java",
        "swt/internal/win32/HIGHCONTRAST.java",
        "swt/internal/win32/ICONINFO.java",
        "swt/internal/win32/INITCOMMONCONTROLSEX.java",
        "swt/internal/win32/INPUT.java",
        "swt/internal/win32/KEYBDINPUT.java",
        "swt/internal/win32/LOGBRUSH.java",
        "swt/internal/win32/LOGFONT.java",
        "swt/internal/win32/LOGFONTA.java",
        "swt/internal/win32/LOGFONTW.java",
        "swt/internal/win32/LOGPEN.java",
        "swt/internal/win32/LVCOLUMN.java",
        "swt/internal/win32/LVHITTESTINFO.java",
        "swt/internal/win32/LVITEM.java",
        "swt/internal/win32/MARGINS.java",
        "swt/internal/win32/MEASUREITEMSTRUCT.java",
        "swt/internal/win32/MENUBARINFO.java",
        "swt/internal/win32/MENUINFO.java",
        "swt/internal/win32/MENUITEMINFO.java",
        "swt/internal/win32/MONITORINFO.java",
        "swt/internal/win32/MOUSEINPUT.java",
        "swt/internal/win32/MSG.java",
        "swt/internal/win32/NMCUSTOMDRAW.java",
        "swt/internal/win32/NMHDR.java",
        "swt/internal/win32/NMHEADER.java",
        "swt/internal/win32/NMLISTVIEW.java",
        "swt/internal/win32/NMLVCUSTOMDRAW.java",
        "swt/internal/win32/NMLVDISPINFO.java",
        "swt/internal/win32/NMLVFINDITEM.java",
        "swt/internal/win32/NMREBARCHEVRON.java",
        "swt/internal/win32/NMRGINFO.java",
        "swt/internal/win32/NMTOOLBAR.java",
        "swt/internal/win32/NMTTDISPINFO.java",
        "swt/internal/win32/NMTTDISPINFOA.java",
        "swt/internal/win32/NMTTDISPINFOW.java",
        "swt/internal/win32/NMTVCUSTOMDRAW.java",
        "swt/internal/win32/NONCLIENTMETRICS.java",
        "swt/internal/win32/NONCLIENTMETRICSA.java",
        "swt/internal/win32/NONCLIENTMETRICSW.java",
        "swt/internal/win32/NOTIFYICONDATA.java",
        "swt/internal/win32/NOTIFYICONDATAA.java",
        "swt/internal/win32/NOTIFYICONDATAW.java",
        "swt/internal/win32/OPENFILENAME.java",
        "swt/internal/win32/OSVERSIONINFO.java",
        "swt/internal/win32/OSVERSIONINFOA.java",
        "swt/internal/win32/OSVERSIONINFOW.java",
        "swt/internal/win32/PAINTSTRUCT.java",
        "swt/internal/win32/POINT.java",
        "swt/internal/win32/PRINTDLG.java",
        "swt/internal/win32/REBARBANDINFO.java",
        "swt/internal/win32/RECT.java",
        "swt/internal/win32/SCRIPT_ANALYSIS.java",
        "swt/internal/win32/SCRIPT_CONTROL.java",
        "swt/internal/win32/SCRIPT_FONTPROPERTIES.java",
        "swt/internal/win32/SCRIPT_ITEM.java",
        "swt/internal/win32/SCRIPT_LOGATTR.java",
        "swt/internal/win32/SCRIPT_PROPERTIES.java",
        "swt/internal/win32/SCRIPT_STATE.java",
        "swt/internal/win32/SCROLLINFO.java",
        "swt/internal/win32/SHACTIVATEINFO.java",
        "swt/internal/win32/SHELLEXECUTEINFO.java",
        "swt/internal/win32/SHMENUBARINFO.java",
        "swt/internal/win32/SHRGINFO.java",
        "swt/internal/win32/SIPINFO.java",
        "swt/internal/win32/SIZE.java",
        "swt/internal/win32/TBBUTTON.java",
        "swt/internal/win32/TBBUTTONINFO.java",
        "swt/internal/win32/TCITEM.java",
        "swt/internal/win32/TEXTMETRIC.java",
        "swt/internal/win32/TEXTMETRICA.java",
        "swt/internal/win32/TEXTMETRICW.java",
        "swt/internal/win32/TOOLINFO.java",
        "swt/internal/win32/TRACKMOUSEEVENT.java",
        "swt/internal/win32/TRIVERTEX.java",
        "swt/internal/win32/TVHITTESTINFO.java",
        "swt/internal/win32/TVINSERTSTRUCT.java",
        "swt/internal/win32/TVITEM.java",
        "swt/internal/win32/WINDOWPLACEMENT.java",
        "swt/internal/win32/WINDOWPOS.java",
        "swt/internal/win32/WNDCLASS.java",
];

/*******************************************************************************
        
        patch replacements, with original and replacement text specified

*******************************************************************************/

SimplePatch[] ImageData = 
[
{"new ImageLoader().load(stream);", "(new ImageLoader()).load(stream);"},
{"new ImageLoader().load(filename);", "(new ImageLoader()).load(filename);"},
{"ANY_TO_EIGHT = new byte[9][];", "ANY_TO_EIGHT = new byte[][9];"},
{"paletteData = new PaletteData(0x0000ff00, 0x00ff0000, 0xff000000);", "paletteData = new PaletteData(0x0000ff00, 0x00ff0000, cast(int) 0xff000000);"},
{"byte [] masks = { (byte)0xFC, (byte)0xF3, (byte)0xCF, (byte)0x3F };", "static byte[] masks = [(byte)0xFC, (byte)0xF3, (byte)0xCF, (byte)0x3F];"},
{imageDataSrc, imageDataRep},
{imageDataSrc1, imageDataRep1},
{imageDataSrc2, imageDataRep2},
{imageDataSrc3, imageDataRep3},
];

SimplePatch[] Compatibility = 
[
{"public static double PI = Math.PI;", "public static const double PI = Math.PI;"},
{"static double toRadians = PI / 180;", "static const double toRadians = PI / 180;"},
];

SimplePatch[] Image = 
[
{"int hdcMask = OS.CreateCompatibleDC(hDC);", "int hdcMask = OS.CreateCompatibleDC(hDC); int hOldSrc, hOldBw, hOldBmp, hb, oldBrush, bm; RECT rect;"},
{"blueMask = 0xFF000000;", "blueMask = cast(int) 0xFF000000;"},
{"\tbm = new BITMAP()", "\tBITMAP bm = new BITMAP()"},
{"new PaletteData(0xFF00, 0xFF0000, 0xFF000000);", "new PaletteData(0xFF00, 0xFF0000, cast(int) 0xFF000000);"},
// optional patches
{"init(device, width, height);", "_init(device, width, height);"},
{"init(device, bounds.width, bounds.height);", "_init(device, bounds.width, bounds.height);"},
{"void init(Device device, int width, int height) {", "void _init(Device device, int width, int height) {"},
{"void init(Device device, ImageData i) {", "void _init(Device device, ImageData i) {"},
{"init(device, srcImage.data);", "_init(device, srcImage.data);"},
{"init(device, data);", "_init(device, data);"},
{"init(device, new ImageData(stream));", "_init(device, new ImageData(stream));"},
{"init(device, new ImageData(filename));", "_init(device, new ImageData(filename));"},
{"init (device, newData);", "_init (device, newData);"},
];

SimplePatch[] Device = 
[
{"public static boolean DEBUG;", "public static const boolean DEBUG;"},
{"scripts = new int [piNumScripts [0]];", "scripts = new int [(piNumScripts [0])];"},
];

SimplePatch[] FileFormat = 
[
{"catch (Exception e) {", "catch (Throwable e) {"},
{"static final String[] FORMATS = {\"WinBMP\", \"WinBMP\", \"GIF\", \"WinICO\", \"JPEG\", \"PNG\", \"TIFF\"};", "static final String[] FORMATS = ConstructArray!(String)(\"WinBMP\", \"WinBMP\", \"GIF\", \"WinICO\", \"JPEG\", \"PNG\", \"TIFF\");"},
];

SimplePatch LEDataInputStream = {"read(buf);", "super.read(buf);"};

SimplePatch OSVERSIONINFO = {"public static /*final*/ int ", "public static int "};

SimplePatch NOTIFYICONDATA = {"final int sizeof = OS.NOTIFYICONDATA_V2_SIZE", "final int sizeof = OS.IsUnicode ? OS.NOTIFYICONDATAW_V2_SIZE : OS.NOTIFYICONDATAA_V2_SIZE;"};

SimplePatch GC = {"int blueMask = 0xFF000000;", "int blueMask = cast(int) 0xFF000000;"};

SimplePatch[] Listener = 
[
{"public interface Listener", "public class Listener"},
{"void handleEvent (Event event)", "private alias void delegate(Event) Delegate; public Delegate handleEvent; this (Delegate handleEvent){this.handleEvent = handleEvent;} this(){}"},
];
       
SimplePatch OleAutomation = {"String[] newNames = new String[count[0]];", "String[] newNames = new String[(count[0])];"};

SimplePatch[] Control = 
[
{"import org.eclipse.swt.accessibility.*;", ""},
{"Accessible accessible;", ""},
{controlSrc1, ""},
{controlSrc2, ""},
{controlSrc3, ""},
{controlSrc4, ""},
];

SimplePatch Table = {"if (accessible == null) accessible = new_Accessible (this);", ""};

SimplePatch[] Layout = 
[
{"protected abstract Point computeSize", "abstract Point computeSize"},
{"protected abstract void layout", "abstract void layout"},
];

SimplePatch[] Synchronizer = 
[
{"protected void syncExec", "void syncExec"},
{"protected void asyncExec", "void asyncExec"},
];

SimplePatch[] Widget = 
[
{"System.out", "System.oot"},
{"int DeferWindowPos(", "Class getClass (){return new Class (this);}\n\nint DeferWindowPos("},
{"data = table = newTable;", "data = (Object) (table = newTable);"},
{"data = table;", "data = (Object) table;"},
{"data = newTable;", "data = (Object) newTable;"},
{"protected void checkWidget () {", "package void checkWidget () {"},
];

SimplePatch[] Display = 
[
{"static final int SWT_GETACCEL", "static const int SWT_GETACCEL"},
{"static final int SWT_KEYMSG", "static const int SWT_KEYMSG"},
{"static final int SWT_DESTROY", "static const int SWT_DESTROY"},
{"static final int SWT_RESIZE", "static const int SWT_RESIZE"},
{"static final int SWT_TRAYICONMSG", "static const int SWT_TRAYICONMSG"},
{"getClass ()", "(new Class(this))"},
{"int [] lpcbData = {128};", "int [] lpcbData = new int[]{128};"},
];

SimplePatch[] TypedEvent = 
[
{"getClass ()", "(new Class(this))"},
];

SimplePatch[] Dialog = 
[
{"getClass ()", "(new Class(this))"},
];

SimplePatch[] Tree = 
[
{"RECT rect2 = rect2 = new RECT ();", "RECT rect2 = new RECT ();"},
{"if (accessible == null) accessible = new_Accessible (this);", ""},
];

SimplePatch[] Menu = 
[
{"if ((hasImage = true) && hasCheck) break;", "hasImage = true; if (hasCheck) break;"},
{"if ((hasCheck = true) && hasImage) break;", "hasCheck = true; if (hasImage) break;"},
{"OS.SendMessage (hwndParent, OS.WM_MENUSELECT, 0xFFFF0000, 0);", "OS.SendMessage (hwndParent, OS.WM_MENUSELECT, (int) 0xFFFF0000, 0);"},
];

SimplePatch RunnableLock = {"void run () {", "void wait(){}\n\nvoid notifyAll(){}\n\nvoid run () {"};

SimplePatch Sash = {"byte [] bits = {-86, 0, 85, 0, -86, 0, 85, 0, -86, 0, 85, 0, -86, 0, 85, 0};", "static byte [] bits = [-86, 0, 85, 0, -86, 0, 85, 0, -86, 0, 85, 0, -86, 0, 85, 0];"};

SimplePatch[] Text1 =
[
{"sendKeyEvent (SWT.KeyUp, OS.WM_IME_CHAR, wParam, lParam);", "super.sendKeyEvent (SWT.KeyUp, OS.WM_IME_CHAR, wParam, lParam);"}, 
{"sendKeyEvent (SWT.KeyDown, OS.WM_IME_CHAR, wParam, lParam)) {", "super.sendKeyEvent (SWT.KeyDown, OS.WM_IME_CHAR, wParam, lParam)) {"}, 
];

SimplePatch Shell = {"return parent != null ? DialogProc : super.windowProc ();", "return parent != null ? DialogProc : Scrollable.windowProc ();"};

SimplePatch FileDialog  = {"String filterPath = \"\", fileName = \"\";", "String filterPath = new String; String fileName = new String;"};

SimplePatch MessageBox = {"String message = \"\";", "String message = new String;"};

SimplePatch Decorations = {"return super.windowProc (hwnd, msg, wParam, lParam);", "return Control.windowProc (hwnd, msg, wParam, lParam);"};


/*******************************************************************************
        
        formatted text replacements

*******************************************************************************/

static const char[] imageDataSrc = 
"}
	static final byte[] ONE_TO_ONE_MAPPING = ANY_TO_EIGHT[8];";

static const char[] imageDataRep = 
"ONE_TO_ONE_MAPPING = ANY_TO_EIGHT[8];
	}
        static final byte[] ONE_TO_ONE_MAPPING;";


static const char[] imageDataSrc1 = 
"static final int
	BLIT_SRC = 1,";

static const char[] imageDataSrc2 = 
"static final int
	ALPHA_OPAQUE = 255,";

static const char[] imageDataSrc3 = 
"private static final int
	// direct / true color formats with arbitrary masks & shifts
	TYPE_GENERIC_8 = 0,";

static const char[] imageDataRep1 = 
"static const int
	BLIT_SRC = 1,";

static const char[] imageDataRep2 = 
"static const int
	ALPHA_OPAQUE = 255,";

static const char[] imageDataRep3 = 
"private static const int
	// direct / true color formats with arbitrary masks & shifts
	TYPE_GENERIC_8 = 0,";


static const char[] controlSrc1 = 
"public Accessible getAccessible () {
	checkWidget ();
	if (accessible == null) accessible = new_Accessible (this);
	return accessible;
}";

static const char[] controlSrc2 = 
"Accessible new_Accessible (Control control) {
	return Accessible.internal_new_Accessible (this);
}";

static const char[] controlSrc3 = 
"if (accessible != null) {
		accessible.internal_dispose_Accessible ();
	}
	accessible = null;";

static const char[] controlSrc4 = 
"if (accessible != null) {
		int result = accessible.internal_WM_GETOBJECT (wParam, lParam);
		if (result != 0) return new LRESULT (result);
	}";


/*******************************************************************************
        
        optional patches for OS.java

*******************************************************************************/

version (OS)
{
static const char[] OSSrc =
"DLLVERSIONINFO dvi = new DLLVERSIONINFO ();
		dvi.cbSize = DLLVERSIONINFO.sizeof;
		dvi.dwMajorVersion = 4;
		TCHAR lpLibFileName = new TCHAR (0, \"Shell32.dll\", true); //$NON-NLS-1$
		int hModule = OS.LoadLibrary (lpLibFileName);";

static const char[] OSRep =
"dvi = new DLLVERSIONINFO ();
		dvi.cbSize = DLLVERSIONINFO.sizeof;
		dvi.dwMajorVersion = 4;
		lpLibFileName = new TCHAR (0, \"Shell32.dll\", true); //$NON-NLS-1$
		hModule = OS.LoadLibrary (lpLibFileName);";


SimplePatch[] OS = 
[
{"static final int SYS_COLOR_INDEX_FLAG = OS.IsWinCE ? 0x40000000 : 0x0;", " static const int SYS_COLOR_INDEX_FLAG = 0x0;"},
{"OSVERSIONINFO.sizeof = info.dwOSVersionInfoSize;", "//OSVERSIONINFO._sizeof = info.dwOSVersionInfoSize;"},
{"public static final native int AbortDoc (int hdc);", "extern (Windows) {\npublic static int AbortDoc (int hdc);"},
{"public static final native int wcslen (int string);", "public static int wcslen (int string);\n}"},
{"HDN_BEGINTRACK = IsUnicode ? 0xfffffeba : 0xfffffece;", "HDN_BEGINTRACK = (IsUnicode ? 0xfffffeba : 0xfffffece);"},
{"HDN_ITEMCHANGED = IsUnicode ? 0xfffffebf : 0xfffffed3", "HDN_ITEMCHANGED = (IsUnicode ? 0xfffffebf : 0xfffffed3);"},
{"TTN_GETDISPINFO = IsUnicode ? 0xfffffdee : 0xfffffdf8;", "TTN_GETDISPINFO = (IsUnicode ? 0xfffffdee : 0xfffffdf8);"},
{"TVN_BEGINDRAG = IsUnicode ? 0xfffffe38 : 0xfffffe69;", "TVN_BEGINDRAG = (IsUnicode ? 0xfffffe38 : 0xfffffe69);"},
{"TVN_BEGINRDRAG = IsUnicode ? 0xfffffe37 : 0xfffffe68;", "TVN_BEGINRDRAG = (IsUnicode ? 0xfffffe37 : 0xfffffe68);"},
{"TVN_ITEMEXPANDING = IsUnicode ? 0xfffffe3a : 0xfffffe6b;", "TVN_ITEMEXPANDING = (IsUnicode ? 0xfffffe3a : 0xfffffe6b);"},
{"TVN_SELCHANGED = IsUnicode ? 0xfffffe3d : 0xfffffe6e;", "TVN_SELCHANGED = (IsUnicode ? 0xfffffe3d : 0xfffffe6e);"},
{"TVN_SELCHANGING = IsUnicode ? 0xfffffe3e : 0xfffffe6f;", "TVN_SELCHANGING = (IsUnicode ? 0xfffffe3e : 0xfffffe6f);"},
{"IsSP()", "_IsSP()"},
{"IsPPC()", "_IsPPC()"},
{OSSrc, OSRep},
];

/*******************************************************************************
        
*******************************************************************************/

class NullAssignment : CustomPatch
{
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "custom null assignments patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"(((char|byte)\s*\[\])[^\?;]+\?) null");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add (" cast(");
                            add (g3);
                            add ("[]) null");
                            eat ();
                            }
                return combine;
        }
}

/*******************************************************************************
        
*******************************************************************************/

class OSConstants : CustomPatch
{
        /***********************************************************************
        
                Return statistics 

        ***********************************************************************/

        char[] stats ()
        {
                return format (count, "custom constants patched");
        }
      
        /***********************************************************************

        ***********************************************************************/

        this ()
        {
                super (r"(final int [\w_]+\s*=\s*)");
        }

        /***********************************************************************

        ***********************************************************************/

        char[] convert (char[] content)
        {
                set (content);
                while (find)
                       with (groups())
                            {
                            ++count;
                            add ();
                            add (g1);
                            add ("cast(int) ");
                            eat ();
                            }
                return combine;
        }
}

}

